==> v1.0.0.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/report/service/impl/ReportBaseInfoServiceImpl.java <==
/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.service.impl;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.springblade.cube.enums.DetailColumnTypeEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springblade.report.mapper.ReportBaseInfoMapper;
import org.springblade.report.service.IReportBaseInfoService;
import org.springblade.report.support.DynamicDataSourceSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.druid.util.JdbcUtils;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 产品订购月报视图 服务实现类
 *
 * @author BladeX
 * @since 2021-03-10
 */
@Slf4j
@Service
//@DS("#session.tenantName")
public class ReportBaseInfoServiceImpl implements IReportBaseInfoService {

	@Autowired
	private ReportBaseInfoMapper reportBaseInfoMapper;
	
//	@Autowired
//	private DynamicRoutingDataSource dynamicRoutingDataSource;

	@Override
//	@DS("#ds.poolName")
	public List<ReportViewDTO> selectReportViewList(ReportDatasource ds) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(ds.getDsType());
		if(dsTypeEnum == DsTypeEnum.sqlserver) {
			return reportBaseInfoMapper.getTableListOfSqlserver(ds.getPoolName());
		}
		List<ReportViewDTO> list = new ArrayList<ReportViewDTO>();
		Connection connection = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = dsTypeEnum == DsTypeEnum.oracle ? ds.getDsUser() : ds.getDsDb();
			ResultSet rs = metaData.getTables(null, schemaPattern, null, new String[] {"TABLE", "VIEW"});
			while(rs.next()) {
				ReportViewDTO viewDto = new ReportViewDTO();
				viewDto.setTableComments(rs.getString("REMARKS"));
				viewDto.setTableViewName(rs.getString("TABLE_NAME"));
				list.add(viewDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}

	@Override
//	@DS("#ds.poolName")
	public List<ReportColumnDTO> selectReportColumnList(ReportDatasource ds, String viewName) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(ds.getDsType());
		if(dsTypeEnum == DsTypeEnum.clickhouse) {
			return getColumnsOfClickhouse(ds, viewName);
		}else if(dsTypeEnum == DsTypeEnum.sqlserver) {
			return getColumnsOfSqlserver(ds, viewName);
		}
		List<ReportColumnDTO> list = new ArrayList<ReportColumnDTO>();
		Connection connection = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = dsTypeEnum == DsTypeEnum.oracle ? ds.getDsUser() : ds.getDsDb();
			ResultSet rs = metaData.getColumns(null, schemaPattern, viewName, "%");
			while(rs.next()) {
				ReportColumnDTO columnDto = new ReportColumnDTO();
				columnDto.setColumnComments(rs.getString("REMARKS"));
				columnDto.setDataType(rs.getString("TYPE_NAME"));
				columnDto.setColumnName(rs.getString("COLUMN_NAME"));
				columnDto.setColumnSize(rs.getInt("COLUMN_SIZE"));
				columnDto.setDecimalDigits(rs.getInt("DECIMAL_DIGITS"));
				columnDto.setColumnType(ViewColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				columnDto.setDetailColumnType(DetailColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				columnDto.setNullAbleStatus("yes".equalsIgnoreCase(rs.getString("IS_NULLABLE")) ? 1 : 0); //IS_NULLABLE - YES/NO  NULLABLE - 1(允许为空)/2(不确定)/0(不允许为空)
				list.add(columnDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}
	
	/**
	 * clickhouse查询列 - 因为clickhouse的列备注不返回，需要特殊处理
	 * @return
	 */
	private List<ReportColumnDTO> getColumnsOfClickhouse(ReportDatasource ds, String viewName){
		List<ReportColumnDTO> list = new ArrayList<ReportColumnDTO>();
		Connection connection = null;
		Statement statement = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds); //dynamicRoutingDataSource.getConnection();
			statement = connection.createStatement();
			ResultSet rs = statement.executeQuery("DESCRIBE TABLE "+ds.getDsDb()+"."+viewName);
			while(rs.next()) {
				ReportColumnDTO columnDto = new ReportColumnDTO();
				String dataType = rs.getString("type");
				columnDto.setColumnComments(rs.getString("comment"));
				columnDto.setDataType(dataType);
				columnDto.setColumnName(rs.getString("name"));
				columnDto.setColumnSize(0);
				columnDto.setDecimalDigits(0);
				columnDto.setNullAbleStatus(StrUtil.containsIgnoreCase(dataType, "nullable") ? 1 : 0);
				if(StrUtil.containsIgnoreCase(dataType, "string")) {
					columnDto.setColumnType(ViewColumnTypeEnum.STRING);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.STRING);
				}else if(StrUtil.containsAnyIgnoreCase(dataType, "date", "time")) {
					columnDto.setColumnType(ViewColumnTypeEnum.DATETIME);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.DATETIME);
				}else if(StrUtil.containsAnyIgnoreCase(dataType, "float", "decimal")) {
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.FLT);
				}else if(StrUtil.containsIgnoreCase(dataType, "bigint")) {
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.LONG);
				}else{
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.INT);
				}
				list.add(columnDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
			JdbcUtils.close(statement);
		}
		return list;
	}
	
	/**
	 * 获取Sqlserver的表字段结构，为什么要单独处理：
	 * 1、SqlserverDataMeata不提供REMARKS(字段注释)；
	 * 2、`sp_columns 表名`，不返回字段注释。
	 * 因此，需要通过SQL查询获取【表注释也需要SQL查询】。
	 * @param ds
	 * @param viewName
	 * @return
	 */
	private List<ReportColumnDTO> getColumnsOfSqlserver(ReportDatasource ds, String viewName){
		List<ReportColumnDTO> columns = reportBaseInfoMapper.getColumnListOfSqlserver(ds.getPoolName(), viewName);
		if(columns != null) {
			columns.stream().forEach(f -> {
				if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "char", "text")) {
					f.setColumnType(ViewColumnTypeEnum.STRING);
					f.setDetailColumnType(DetailColumnTypeEnum.STRING);
				}else if(StrUtil.containsIgnoreCase(f.getDataType(), "date")) {
					f.setColumnType(ViewColumnTypeEnum.DATETIME);
					f.setDetailColumnType(DetailColumnTypeEnum.DATETIME);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "float", "double", "real", "decimal")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.FLT);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "int64")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.LONG);
				}else{
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.INT);
				}
			});
		}
		return columns;
	}
}

==> v1.1.1.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/report/service/impl/ReportBaseInfoServiceImpl.java <==
/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.service.impl;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.springblade.cube.enums.DetailColumnTypeEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springblade.report.mapper.ReportBaseInfoMapper;
import org.springblade.report.service.IReportBaseInfoService;
import org.springblade.report.support.DynamicDataSourceSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.druid.util.JdbcUtils;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 产品订购月报视图 服务实现类
 *
 * @author BladeX
 * @since 2021-03-10
 */
@Slf4j
@Service
//@DS("#session.tenantName")
public class ReportBaseInfoServiceImpl implements IReportBaseInfoService {

	@Autowired
	private ReportBaseInfoMapper reportBaseInfoMapper;
	
//	@Autowired
//	private DynamicRoutingDataSource dynamicRoutingDataSource;

	@Override
//	@DS("#ds.poolName")
	public List<ReportViewDTO> selectReportViewList(ReportDatasource ds) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(ds.getDsType());
		if(dsTypeEnum == DsTypeEnum.sqlserver) {
			return reportBaseInfoMapper.getTableListOfSqlserver(ds.getPoolName());
		}
		List<ReportViewDTO> list = new ArrayList<ReportViewDTO>();
		Connection connection = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = _dbSchemaPattern(ds);//dsTypeEnum == DsTypeEnum.oracle ? ds.getDsUser() : ds.getDsDb();
			ResultSet rs = metaData.getTables(null, schemaPattern, null, new String[] {"TABLE", "VIEW"});
			while(rs.next()) {
				ReportViewDTO viewDto = new ReportViewDTO();
				viewDto.setTableComments(rs.getString("REMARKS"));
				viewDto.setTableViewName(rs.getString("TABLE_NAME"));
				list.add(viewDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}

	@Override
//	@DS("#ds.poolName")
	public List<ReportColumnDTO> selectReportColumnList(ReportDatasource ds, String viewName) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(ds.getDsType());
		if(dsTypeEnum == DsTypeEnum.clickhouse) {
			return getColumnsOfClickhouse(ds, viewName);
		}else if(dsTypeEnum == DsTypeEnum.sqlserver) {
			return getColumnsOfSqlserver(ds, viewName);
		}
		List<ReportColumnDTO> list = new ArrayList<ReportColumnDTO>();
		Connection connection = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = _dbSchemaPattern(ds);//dsTypeEnum == DsTypeEnum.oracle ? ds.getDsUser() : ds.getDsDb();
			ResultSet rs = metaData.getColumns(null, schemaPattern, viewName, "%");
			while(rs.next()) {
				ReportColumnDTO columnDto = new ReportColumnDTO();
				columnDto.setColumnComments(rs.getString("REMARKS"));
				columnDto.setDataType(rs.getString("TYPE_NAME"));
				columnDto.setColumnName(rs.getString("COLUMN_NAME"));
				columnDto.setColumnSize(rs.getInt("COLUMN_SIZE"));
				columnDto.setDecimalDigits(rs.getInt("DECIMAL_DIGITS"));
				columnDto.setColumnType(ViewColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				columnDto.setDetailColumnType(DetailColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				columnDto.setNullAbleStatus("yes".equalsIgnoreCase(rs.getString("IS_NULLABLE")) ? 1 : 0); //IS_NULLABLE - YES/NO  NULLABLE - 1(允许为空)/2(不确定)/0(不允许为空)
				list.add(columnDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}
	
	/**
	 * clickhouse查询列 - 因为clickhouse的列备注不返回，需要特殊处理
	 * @return
	 */
	private List<ReportColumnDTO> getColumnsOfClickhouse(ReportDatasource ds, String viewName){
		List<ReportColumnDTO> list = new ArrayList<ReportColumnDTO>();
		Connection connection = null;
		Statement statement = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds); //dynamicRoutingDataSource.getConnection();
			statement = connection.createStatement();
			ResultSet rs = statement.executeQuery("DESCRIBE TABLE "+ds.getDsDb()+"."+viewName);
			while(rs.next()) {
				ReportColumnDTO columnDto = new ReportColumnDTO();
				String dataType = rs.getString("type");
				columnDto.setColumnComments(rs.getString("comment"));
				columnDto.setDataType(dataType);
				columnDto.setColumnName(rs.getString("name"));
				columnDto.setColumnSize(0);
				columnDto.setDecimalDigits(0);
				columnDto.setNullAbleStatus(StrUtil.containsIgnoreCase(dataType, "nullable") ? 1 : 0);
				if(StrUtil.containsIgnoreCase(dataType, "string")) {
					columnDto.setColumnType(ViewColumnTypeEnum.STRING);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.STRING);
				}else if(StrUtil.containsAnyIgnoreCase(dataType, "date", "time")) {
					columnDto.setColumnType(ViewColumnTypeEnum.DATETIME);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.DATETIME);
				}else if(StrUtil.containsAnyIgnoreCase(dataType, "float", "decimal")) {
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.FLT);
				}else if(StrUtil.containsIgnoreCase(dataType, "bigint")) {
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.LONG);
				}else{
					columnDto.setColumnType(ViewColumnTypeEnum.NUMBER);
					columnDto.setDetailColumnType(DetailColumnTypeEnum.INT);
				}
				list.add(columnDto);
			}
			rs.close();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			JdbcUtils.close(connection);
			JdbcUtils.close(statement);
		}
		return list;
	}
	
	/**
	 * 获取Sqlserver的表字段结构，为什么要单独处理：
	 * 1、SqlserverDataMeata不提供REMARKS(字段注释)；
	 * 2、`sp_columns 表名`，不返回字段注释。
	 * 因此，需要通过SQL查询获取【表注释也需要SQL查询】。
	 * @param ds
	 * @param viewName
	 * @return
	 */
	private List<ReportColumnDTO> getColumnsOfSqlserver(ReportDatasource ds, String viewName){
		List<ReportColumnDTO> columns = reportBaseInfoMapper.getColumnListOfSqlserver(ds.getPoolName(), viewName);
		if(columns != null) {
			columns.stream().forEach(f -> {
				if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "char", "text")) {
					f.setColumnType(ViewColumnTypeEnum.STRING);
					f.setDetailColumnType(DetailColumnTypeEnum.STRING);
				}else if(StrUtil.containsIgnoreCase(f.getDataType(), "date")) {
					f.setColumnType(ViewColumnTypeEnum.DATETIME);
					f.setDetailColumnType(DetailColumnTypeEnum.DATETIME);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "float", "double", "real", "decimal")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.FLT);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "int64")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.LONG);
				}else{
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.INT);
				}
			});
		}
		return columns;
	}
	
	private String _dbSchemaPattern(ReportDatasource ds) {
		if(ds.getDsTypeSyl() == DsTypeEnum.oracle) {
			return ds.getDsUser();
		}else if(ds.getDsTypeSyl() == DsTypeEnum.kingbase || ds.getDsTypeSyl() == DsTypeEnum.postgresql) {
			return "public";
		}
		return ds.getDsDb();
	}
}
